import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:dshell/dshell.dart';
import 'package:uuid/uuid.dart';

class BuildCommand extends Command<void> {
  @override
  String get description => 'Builds your nginx container';

  @override
  String get name => 'build';

  BuildCommand() {
    argParser.addOption('image',
        abbr: 'i',
        help: 'The docker image name in the form --image="repo/image:version"');

    argParser.addFlag('update-dshell',
        abbr: 'u',
        help:
            'Pass this flag to force the build to pull the latest version of dart/dshell',
        defaultsTo: false);
    argParser.addFlag('debug',
        abbr: 'd', help: 'Outputs additional build information');
  }
  @override
  void run() {
    var results = argResults;

    var image = argResults['image'] as String;

    if (image == null) {
      print('You must pass a image --image=repo/image:version');
      showUsage(argParser);
    }

    var pulldshell = results['update-dshell'] as bool;

    /// force dshell to pull the latest version.
    if (pulldshell || !exists('update-dshell.txt')) {
      'update-dshell.txt'.write(Uuid().v4());
    }

    print(blue('Building nginx-le $image '));

    /// required to give docker access to our ssh keys.
    'docker build -t $image .'.run;
  }

  void showUsage(ArgParser parser) {
    print(parser.usage);
    exit(-1);
  }
}
