import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:dshell/dshell.dart';
import 'package:nginx_le_cli/src/external_run_config.dart';
import 'package:nginx_le_shared/nginx_le_shared.dart';

/// Starts nginx and the certbot scheduler.
class RestartCommand extends Command<void> {
  @override
  String get description =>
      'starts the ngix server after it has been started at least once';

  @override
  String get name => 'restart';

  RestartCommand() {
    argParser.addFlag('debug',
        abbr: 'd', help: 'Outputs additional logging information');
  }

  @override
  void run() {
    print('nginx-le container is starting.');

    if (!ExternalRunConfig.exists) {
      print("A saved configuration doesn't exist. You must use start.");
      showUsage(argParser);
    }
    var config = ExternalRunConfig.load();

    if (!Images().existsByImageId(imageid: config.image)) {
      print('The ngnix-le image ${config.image} no longer exists');
      exit(1);
    }

    var debug = argResults['debug'] as bool;
    debug ??= false;

    print('Restarting nginx fqdn=${config.fqdn} mode=${config.mode}');
    var containerName = '';
    if (config.name != null) {
      containerName = '--name=${config.name}';
    }
    'docker run'
            ' $containerName'
            ' --env=FQDN=${config.fqdn}'
            ' --env=TLD=${config.tld}'
            ' --env=MODE=${config.mode}'
            ' --env=EMAIL_ADDRESS=${config.emailaddress}'
            ' --env=DEBUG=$debug'
            ' --net=host'
            ' -v certificates:${Certbot.letsEncryptRootPath}'
            ' ${config.image}'
        .run;
  }

  void showUsage(ArgParser parser) {
    print(parser.usage);
    exit(-1);
  }
}
