import 'package:args/command_runner.dart';
import 'package:dshell/dshell.dart';

import 'util.dart';

class RevokeCommand extends Command<void> {
  @override
  String get description =>
      'Revokes the active Lets Encrypt certificate and places the server into acquire mode.';

  @override
  String get name => 'revoke';

  RevokeCommand() {
    argParser.addOption('containerid',
        abbr: 'c',
        help:
            'The docker containerid to attach to in the form --containerid="XXXXX"');
    argParser.addOption('name',
        abbr: 'n', help: 'The name of the docker container to attach to');
    argParser.addFlag('debug',
        abbr: 'd',
        defaultsTo: false,
        help: 'Outputs additional logging information');
    argParser.addFlag('staging',
        abbr: 's',
        defaultsTo: false,
        help: 'Revokes a staging (test) certificate');
  }

  @override
  void run() {
    print('revoke command running');

    var staging = argResults['staging'] as bool;
    var debug = argResults['debug'] as bool;

    var target = containerOrName(argParser, argResults);

    var cmd = 'docker exec -it $target /home/bin/revoke';
    if (staging) cmd += ' --staging';
    if (debug) cmd += ' --debug';
    cmd.run;
  }
}
