import 'package:dshell/dshell.dart' hide exists;
import 'package:dshell/dshell.dart' as d;
import 'package:meta/meta.dart';

/// Used to store the variables that the container was started
/// with.
/// This allows subsequent commands run agains the container
/// to access the startup variables.
class ExternalRunConfig {
  String fqdn;
  String tld;
  String image;
  String name; // container name
  String mode;
  String emailaddress;
  // String namecheap_apikey;
  // String namecheap_apiuser;

  static const savefile = '.nginx-le';
  ExternalRunConfig(
      {@required this.fqdn,
      @required this.tld,
      @required this.image,
      @required this.name,
      @required this.emailaddress,
      @required this.mode});

  // bool get hasCredentials =>
  //     namecheap_apikey != null &&
  //     namecheap_apikey.isNotEmpty &&
  //     namecheap_apiuser != null &&
  //     namecheap_apiuser.isNotEmpty;

  // void setcredentials(String namecheap_apikey, String namecheap_apiuser) {
  //   load();
  //   this.namecheap_apikey = namecheap_apikey;
  //   this.namecheap_apiuser = namecheap_apiuser;
  //   save();
  // }

  void save() {
    var savefilepath = join(HOME, savefile);
    savefilepath.write('version:1');
    savefilepath.append('fqdn:${fqdn.trim()}');
    savefilepath.append('tld:${tld.trim()}');
    savefilepath.append('emailaddress:${emailaddress.trim()}');
    savefilepath.append('mode:${mode.trim()}');
    savefilepath.append('image:${image.trim()}');
    savefilepath.append('name:${name?.trim()}');
    // savefilepath.append('namecheap_apikey:${namecheap_apikey.trim()}');
    // savefilepath.append('namecheap_apiuser:${namecheap_apiuser.trim()}');
  }

  static bool get exists => d.exists(join(HOME, savefile));

  static ExternalRunConfig load() {
    var savefilepath = join(HOME, savefile);
    var lines = read(savefilepath).toList();

    var version = lines[0];
    var fqdn = lines[1];
    var tld = lines[2];
    var emailAddress = lines[3];
    var mode = lines[4];
    var image = lines[5];
    var name = lines[6];
    // var namecheap_apikey = lines[5];
    // var namecheap_apiuser = lines[6];

    if (version != 'version:1') {
      throw ArgumentError.value(version, 'Invalid version');
    }

    if (!fqdn.startsWith('fqdn')) {
      throw ArgumentError.value(fqdn, 'Expected fqdn');
    }

    if (!tld.startsWith('tld')) {
      throw ArgumentError.value(fqdn, 'Expected tld');
    }

    if (!emailAddress.startsWith('emailaddress')) {
      throw ArgumentError.value(emailAddress, 'Expected emailaddress');
    }

    if (!mode.startsWith('mode')) {
      throw ArgumentError.value(mode, 'Expected mode');
    }

    if (!image.startsWith('image')) {
      throw ArgumentError.value(image, 'Expected image');
    }

    if (!name.startsWith('name')) {
      throw ArgumentError.value(name, 'Expected name');
    }

    // if (!mode.startsWith('namecheap_apikey')) {
    //   throw ArgumentError.value(namecheap_apikey, 'Expected namecheap_apikey');
    // }

    // if (!mode.startsWith('namecheap_apiuser')) {
    //   throw ArgumentError.value(
    //       namecheap_apiuser, 'Expected namecheap_apiuser');
    // }

    fqdn = fqdn.split(':')[1];
    tld = tld.split(':')[1];
    emailAddress = emailAddress.split(':')[1];
    mode = mode.split(':')[1];
    image = image.split(':')[1];
    name = name.split(':')[1];
    // namecheap_apikey = namecheap_apikey.split(':')[1];
    // namecheap_apiuser = namecheap_apiuser.split(':')[1];

    return ExternalRunConfig(
      fqdn: fqdn,
      tld: tld,
      emailaddress: emailAddress,
      mode: mode,
      image: image,
      name: name,
    );
  }
}
